# -*- coding: utf-8 -*-
# compatibility layer between saas-n and saas-n+1
import passlib

from odoo import http

__all__ = ['db_monodb', 'route', 'RegistryManager']


def db_monodb(req):
    if hasattr(http, 'db_monodb'):
        # < saas-15.3
        return http.db_monodb(req.httprequest)
    return req.db

route = http.route
request = http.request


def request_type(req=request):
    if hasattr(req, '_request_type'):
        # < saas-15.3
        return req._request_type
    return req.dispatcher.routing_type


def request_jsonrequest(req=request):
    if hasattr(req, 'jsonrequest'):
        # < saas-15.3
        return req.jsonrequest
    return req.dispatcher.jsonrequest


from odoo.service.db import exp_list_lang
ALL_LANGUAGES = {l[0] for l in exp_list_lang()}

# >= saas-16
# https://github.com/odoo/internal/commit/13f7f6fd23e680e3b183439fffaa0426d5eac769
# https://github.com/odoo/internal/commit/fd12002bfb36db548f765463ff9d880dd54915df
from odoo.modules.registry import Registry

class RegistryManager(object):
    @classmethod
    def new(cls, db_name, update_module=False):
        return Registry.new(db_name, update_module=update_module)

    @classmethod
    def get(cls, db_name):
        return Registry(db_name)

    @classmethod
    def delete(cls, db_name):
        return Registry.delete(db_name)

    @classmethod
    def signal_registry_change(cls, dbname):
        cls.get(dbname).signal_changes()

try:
    from odoo.api import multi
except ImportError:
    # >= 13.0
    from odoo import api

    def api_noop(f):
        return f
    api.multi = api_noop
    api.model_cr = api_noop


def _consteq(str1, str2):
    """ Constant-time string comparison. Suitable to compare bytestrings of fixed,
        known length only, because length difference is optimized. """
    return len(str1) == len(str2) and sum(ord(x)^ord(y) for x, y in zip(str1, str2)) == 0

consteq = getattr(passlib.utils, 'consteq', _consteq) if hasattr(passlib, 'utils') else _consteq


# load_information_from_description_file renamed and moved
try:
    # >= 15.2
    from odoo.modules.module import load_manifest as load_manifest
except ImportError:
    from odoo.modules import load_information_from_description_file as load_manifest

try:
    # >= 14.0
    from odoo.addons.base.models.res_users import check_identity
except ImportError:
    def check_identity(fn):
        return fn
